from typing import List

from .card import BriscolaCard
from .deck import BriscolaDeck
from .player import BriscolaPlayer, Player


class Game():
    def __init__(self) -> None:
        pass


class BriscolaGame(Game):

    def __init__(self, players: List[BriscolaPlayer]):
        self.deckobject = BriscolaDeck()
        self.players = players
        self.lenplayers = len(players)
        self.trumpcard = self.deckobject.deck[-1]
        print("The trump card is {}\n".format(str(self.trumpcard)))
        self.trump = self.trumpcard.color
        BriscolaDeck.reorder(self.trump)
        self.startgame()
        self.playaparty()

    def startgame(self) -> None:
        for _ in range(3):
            self.alldraw()
        # store state

    def calculate_whowin(self, cardplayed: List[BriscolaCard]) -> int:
        indbestcard = 0
        asked = cardplayed[0]
        for i, c in enumerate(cardplayed):
            if c.color == self.trump:
                if asked.color != self.trump or c.gamevalue >= asked.gamevalue:
                    asked = c
            if asked.color == c.color and c.gamevalue >= asked.gamevalue:
                indbestcard = i
                asked = c
        return indbestcard

    def playoneturn(self) -> None:
        cardplayed = []
        for p in self.players:
            # store state
            cardplayed.append(p.playcard(self))
        whowin = self.calculate_whowin(cardplayed)
        self.players[whowin].winnedcards.extend(cardplayed)
        print('{} win the turn!'.format(self.players[whowin].name))
        self.players = self.players[whowin:] + self.players[:whowin]

    def stoptheparty(self) -> bool:
        deck = self.deckobject.deck
        lendeck = len(deck)
        first_hand = self.players[0].hand
        if (lendeck == lendeck % self.lenplayers) and first_hand == []:
            Player.resetNumber()
            # store state
            if deck != []:
                self.players[0].winnedcards.append(deck.pop(0))
                self.players[0].calculscore()
            return False
        else:
            return True

    def alldraw(self) -> None:
        for p in self.players:
            p.drawcard(self.deckobject.deck)

    def allscore(self) -> None:
        for p in self.players:
            p.calculscore()

    def printenscore(self) -> None:
        self.players.sort(key=lambda x: x.score, reverse=True)
        print("{} win with {}".format(
            self.players[0].name, self.players[0].score))
        print("\n".join(("{}\t{}".format(p.name, p.score)
                         for p in self.players[1:])))

    def playaparty(self) -> None:
        deck = self.deckobject.deck
        while self.stoptheparty():
            self.playoneturn()
            lendeck = len(deck)
            if (lendeck != lendeck % self.lenplayers):
                self.alldraw()
            self.allscore()
            print()
        self.printenscore()



import random
import string
from typing import List

from .card import BriscolaCard, Card
from .interactive import InteractiveBriscola


class Player():

    number: int = 0

    @staticmethod
    def numberPlayer() -> int:
        Player.number += 1
        return Player.number - 1

    @staticmethod
    def resetNumber() -> None:
        Player.number = 0

    def __init__(self, name: str = ''):
        self.hand: List[Card] = []
        self.score: int = 0
        self.name: str = name
        self.number: int = Player.numberPlayer()
        if name == "":
            self.name = ''.join(random.choice(string.ascii_lowercase)
                                for i in range(3))
        self.name += ' ' + str(self.number)
        self.winnedcards: List[Card] = []
        self.chosencard: Card = None

    def drawcard(self, deck: List[Card]) -> str:
        drawed_card: Card = deck.pop(0)
        self.hand.append(drawed_card)
        return str(drawed_card)

    def playcard(self, game=None):
        self.chosencard = self.hand.pop(0)
        print("{name}\t({score}) played:\t{card}".format(name=self.name,
                                                         score=self.score,
                                                         card=self.chosencard))
        return self.chosencard

    def calculscore(self) -> None:
        self.score = sum((c.score for c in self.winnedcards))


class BriscolaPlayer(Player):

    def __init__(self, name: str, auto: bool = True):
        super().__init__(name)
        self.auto = auto
        # self.vector = BriscolaPlayerVector(self.number)

    def playcard(self, game=None) -> BriscolaCard:
        if not self.auto:
            InteractiveBriscola(self, game).cmdloop(
                intro='Your cards: ' + ', '.join(("{}: {}".format(i + 1, x)
                                                  for i, x in enumerate(
                                                  self.getstrhand()))))
        return super().playcard()

    def getstrhand(self) -> List[str]:
        return list(map(str, self.hand))


class ComputerBriscolaPlayer(Player):
    def __init__(self) -> None:
        super().__init__()
        self.name = "cmp_" + self.name



import itertools
import random
from typing import List

from .card import Card, BriscolaCard


class Deck():
    def __init__(self, values: List[str],
                 colors: List[str], cardobject: type = Card) -> None:
        self.deck: List[Card] = [cardobject(v, c)
                                 for c, v in itertools.product(colors, values)]

    def shuffle(self) -> None:
        random.shuffle(self.deck)

    def __str__(self) -> str:
        return "\n".join((str(c) for c in self.deck))


class BriscolaDeck(Deck):
    colors = ['Gold', 'WoodenStick', 'Sword', 'Cup']
    values = (("As", 10, 11),
              ("3", 9, 10),
              ("King", 8, 4),
              ("Horseman", 7, 3),
              ("Jack", 6, 2),
              ("7", 5, 0),
              ("6", 4, 0),
              ("5", 3, 0),
              ("4", 2, 0),
              ("2", 1, 0))
    ordereddeck: List[BriscolaCard] = []

    def __init__(self) -> None:
        super().__init__(BriscolaDeck.values, BriscolaDeck.colors,
                         BriscolaCard)
        BriscolaDeck.ordereddeck = self.deck.copy()
        self.shuffle()

    @staticmethod
    def reorder(trump: str) -> None:
        bod = BriscolaDeck.ordereddeck
        lenVal = len(BriscolaDeck.values)
        start = BriscolaDeck.colors.index(trump) * lenVal
        end = start + lenVal
        bod[0: lenVal], bod[start: end] = bod[start: end], bod[0: lenVal]



class Card():

    def __init__(self, value, color):
        self.color: str = color
        self.value: str = value

    def __str__(self):
        return "{v} of {c}".format(v=self.value, c=self.color)


class BriscolaCard(Card):
    def __init__(self, value: str, color: str):
        super().__init__(value, color)
        self.value, self.gamevalue, self.score = self.value



import cmd


class Interactive(cmd.Cmd):
    """docstring for ."""

    def __init__(self):
        super().__init__()


class InteractiveBriscola(Interactive):
    completekey = 'tab'

    def __init__(self, actual_player, game):
        super().__init__()
        self.player = actual_player
        self.cards = self.player.getstrhand()
        self.cardsdisplay = ', '.join(
            ["{}: {}".format(i + 1, c) for i, c in enumerate(self.cards)])
        self.game = game
        InteractiveBriscola.prompt = actual_player.name + ' >'

    def do_trump(self):
        """Display the trump card."""
        print("The trump card is {}\n".format(str(self.game.trumpcard)))

    def do_card(self, arg):
        """Display player's cards separate with a comma."""
        print(self.cardsdisplay)

    def do_play(self, arg=0):
        """play [cardname|numcard]\nPick up a card to play."""
        if arg.replace(' ', ' ') in self.cards or (
                arg.isdigit() and 0 < int(arg) <= len(self.cards)):
            indchosen = self.cards.index(
                arg) if not arg.isdigit() else int(arg) - 1
            self.player.hand[0], self.player.hand[
                indchosen] = self.player.hand[indchosen], self.player.hand[0]
            return True
        else:
            print("Please chose a card in your hand.\n", self.cardsdisplay)

    def complete_play(self, text, line, begidx, endidx):
        if text:
            return [f for f in self.cards if f.startswith(text)]
        return self.cards

    def do_exit(self, line):
        """Exit the game (Your progression will be lost)."""
        exit()
        return True



#!/usr/bin/env python3

import os
import random

around = os.listdir()
if 'gameofcard' in around:
    import gameofcard.gameobject
    go = gameofcard.gameobject
elif 'gameobject' in around:
    import gameobject
    go = gameobject
else:
    import gameobject
    go = gameobject

if __name__ == '__main__':
    random.seed(1)
    game = go.BriscolaGame(
        [go.BriscolaPlayer("East"),
         go.BriscolaPlayer("North"),
         go.BriscolaPlayer("West"),
         go.BriscolaPlayer("Me", False)
         ])



"""Je suis une docstring."""
from .card import BriscolaCard, Card
from .deck import BriscolaDeck, Deck
from .game import BriscolaGame, Game
from .interactive import Interactive, InteractiveBriscola
from .player import BriscolaPlayer, ComputerBriscolaPlayer, Player
from .playervector import BriscolaPlayerVector, PlayerVector

__all__ = ['Card', 'BriscolaCard', 'Deck', 'BriscolaDeck', 'Game',
           'BriscolaGame', 'Interactive', 'InteractiveBriscola',
           'BriscolaPlayer', 'ComputerBriscolaPlayer', 'Player',
           'BriscolaPlayerVector', "PlayerVector"]



#!/usr/bin/env python3

from setuptools import find_packages, setup

import gameofcard as gc

URL = 'https://github.com/EPgg92/gameofcard/archive/{}'.format(gc.VERSION)

# with open("README.md") as target:
#     long_description = target.read()

setup(
    #     name='gameofcard',
    packages=find_packages(),
    #     version=VERSION,
    #     description='Just a humble implementation of different card games.',
    #     long_description=long_description,
    long_description_content_type='text/markdown',
    #     author='EPgg92',
    #     author_email='poggio.enzo@gmail.com',
    url=URL,
    download_url=URL + '.tar.gz',
    #     keywords=['card', 'game'],
    #     classifiers=["Programming Language :: Python :: 3",
    #                  "Operating System :: OS Independent", ],
)



from typing import List

from .deck import Deck, BriscolaDeck


class PlayerVector():
    def __init__(self, gameDeck: Deck, playernumber: int):
        self.playernumber = playernumber
        self.gameDeck = gameDeck
        self.cards: List[int] = []


class BriscolaPlayerVector(PlayerVector):
    def __init__(self, playernumber: int = 0) -> None:
        super().__init__(BriscolaDeck, playernumber)
        self.scores = []
        self.played = []
